from time import sleep
from queue import Queue
from threading import Thread


class MessageQueueHandler:
    keys = ['message.queue']
    type_id = 'ControlEventHandler'
    avg_word_len = 5

    def __init__(self, wpm):
        self.connector = None
        self.wpm = wpm
        self._queue = Queue()
        self._thread = Thread(target=self._dispatch_messages)

    def _dispatch_messages(self):
        for message in iter(self._queue.get, None):
            message['typing'] = not self._queue.empty()
            self.connector.send_message(message)
            self._sleep(self._get_sleep_time(message))

    def _get_sleep_time(self, message):
        words_per_min = message['wpm'] if 'wpm' in message else self.wpm
        chars_per_min = self.avg_word_len * words_per_min / 60
        return len(message['message']) / chars_per_min

    def _sleep(self, seconds):  # pylint: disable=no-self-use
        sleep(seconds)

    def handle_event(self, message, _):
        for unpacked in self._generate_messages_from_queue(message):
            self._queue.put(unpacked)

    @staticmethod
    def _generate_messages_from_queue(queue_message):
        for message in queue_message['messages']:
            yield {
                'key': 'message.send',
                **message
            }

    def start(self):
        self._thread.start()

    def cleanup(self):
        self._queue.queue.clear()
        self._queue.put(None)
        self._thread.join()
